{%- from "macros/form.html" import  m_field with context %}
{%- for field in form %} {# form could also be a formfield instance #}
  <div class="col-xs-12 {%- if field.flags.hidden %} hide{%- endif %}">
    {{ m_field(field, label_width=3, field_width=9) }}
  </div>
{%-  endfor %}
