{% macro m_recent_items(recent_items) %}
  {%- if recent_items %}
    <div class="well" style="padding: 8px 0;">
      <ul class="nav nav-list">
        <li class="nav-header">{{ _("Recent items") }}</li>
        {%- for item in recent_items %}
          <li class="recent-item">
            <a href="{{ item.url }}">
              <img alt=""
                  src="/static/icons/{{ item.type }}-12.png"/>&nbsp;{{ item.name }}
            </a>
          </li>
        {%- endfor %}
      </ul>
    </div>
  {%- endif %}
{% endmacro %}
